# eslint-config-osedea

## Peer dependencies

`eslint >= 5.16.0` is a requirement. `Flow-bin`, `Prettier` & `Typescript` are requirements for certain configs as listed below.


## Configuring rules for project

`eslint-config-osedea` comes with the following environments, some of which are already pre-configured, however you can mix-and-match the environments if you want.

Config name   | Extends    | Dependencies (besides eslint)
------------- | ---------- | ------
angular       |            |
es5           |            |
esnext        | es5        |
flow          |            | flow-bin
node          |            |
prettier      |            | prettier
react         | esnext     |
react-native  | react      |
testing       |            |
typescript    | esnext     | typescript

Create a file called `.eslintrc.yml` in the root of your project. Here, you will specify your rules, and extend one (or more) of the above rulesets.

Some example `.eslintrc.yml` configs:

```yml
extends:
  - osedea/react-native
  - osedea/typescript
  - osedea/testing
  - osedea/prettier # always needs to be last
```

```yml
extends:
  - osedea/es5
  - osedea/angular
  - osedea/testing

# You can override rules from the linter here. Use at your own risk.
# rules:
#   comma-dangle: off # For example, if you do not want the linter to check for dangling commas ever
```

## Global settings

Depending on your config, you may need to add a settings block to your `.eslintrc.yml`.

### osedea/flow

It is recommended to install `eslint-plugin-flowtype-errors` in your project directly.

### osedea/node

For `node` config, it uses the `engines` section of `package.json`:
```json
{
  ...
  engines: {
    "node": ">= 6.0.0"
  },
  ...
}
```

### osedea/prettier

Needs to be added last so it is able to override other configs.

### osedea/react

For `react` config (if version is not specified, will assume latest version):

```yml
settings:
  react:
    version: "detect"
    flowVersion: "0.86" # for example. can omit line if not using flow
```

### osedea/react-native

For `react-native` config, in order to resolve imports (import x from CoolProject/.../...), the project name must be defined. The plugin will use the `name` param of the package.json in order to solve this.
```json
{
  "name": "CoolProject",
  ...
}
```

### osedea/typescript

If using VSCode & eslint errors aren't displaying, add this to your `settings.json`:

```json
    "eslint.validate": [
        "javascript",
        "javascriptreact",
        { "language": "typescript", "autoFix": true },
        { "language": "typescriptreact", "autoFix": true }
    ],
```
